qsub -t 1-3419 bulk_qsub_self_rnaplex.sh /scr/k70san/fall/RINTERwahn/PARIS/Annotated/Beds/SplitFasta/ /scr/k70san/fall/RINTERwahn/PARIS/Annotated/Beds/PlFold_Indexed/ /scr/k70san/fall/RINTERwahn/PARIS/PlexOut/
